import React, { Component } from 'react';
import './Scroller.scss';

export interface ScrollerProps {
  height?: string,
}

interface Elements {
  thumb?: HTMLElement
  content?: HTMLElement
  wrapper?: HTMLElement
}

interface State {
  pixelRatio: number
  contentHeight: number
  wrapperHeight: number
}

export default class Scroller extends Component<ScrollerProps>{

  public state: State = {
    pixelRatio: 0,
    contentHeight: 0,
    wrapperHeight: 0,
  }

  private el: Elements = {}
  private mouse = {
    y: 0
  }

  constructor(props) {
    super(props)
  }

  public componentDidMount() {
    setTimeout(() => {
      const contentHeight = this.el.content.offsetHeight
      const wrapperHeight = this.el.wrapper.offsetHeight
      this.setState({
        contentHeight,
        wrapperHeight,
        pixelRatio: wrapperHeight / contentHeight,
      })
    }, 100)
  }

  public render() {
    return (
      <div className="scroll-wrapper" ref={el => this.el.wrapper = el}
        style={{ height: this.props.height || '100%' }}>
        <div className="scroll-content-area">
          <div className="scroll-content" ref={el => this.el.content = el}>
            {this.props.children}
          </div>
        </div>
        <div className={"scrollbar "}>
          <div className="scrollbar-thumb"
            style={{ height: this.state.pixelRatio * this.state.wrapperHeight }}
            onMouseDown={this.onThumbMouseDown} ref={el => this.el.thumb = el}></div>
        </div>
      </div>
    )
  }

  private onThumbMouseDown = (e) => {
    this.mouse.y = e.pageY
    document.addEventListener('mousemove', this.onMouseMove)
    document.addEventListener('mouseup', this.onMouseUp)
  }

  private onMouseMove = (e) => {
    const delta = e.pageY - this.mouse.y
    this.mouse.y = e.pageY
    const thumbTop = parseFloat(this.el.thumb.style.top || '0') + delta
    const contentTop = parseFloat(this.el.content.style.top || '0') - delta / this.state.pixelRatio
    if (contentTop >= -(this.state.contentHeight - this.state.wrapperHeight) && contentTop <= 0) {
      this.el.thumb.style.top = `${thumbTop}px`;
      this.el.content.style.top = `${contentTop}px`;
    }
  }

  private onMouseUp = (e) => {
    document.removeEventListener('mousemove', this.onMouseMove)
  }

}
